CONFIG_FILE_NAME = 'config_v5'

PREFERENCES = 'p'
BASE_URL = 'u'
REDIRECT = 'r'

FEATURE_FLAGS = 'f'
VALUE = 'v'
COMPARATOR = 't'
COMPARISON_ATTRIBUTE = 'a'
COMPARISON_VALUE = 'c'
ROLLOUT_PERCENTAGE_ITEMS = 'p'
PERCENTAGE = 'p'
ROLLOUT_RULES = 'r'
VARIATION_ID = 'i'
